<script>import { createEventDispatcher, onMount } from 'svelte';
import { fade, scale } from 'svelte/transition';
import { sineInOut } from 'svelte/easing';
import { focustrap, lockscroll, useFocusReturn } from '@svelteuidev/composables';
import { getTransition } from '../../internal';
import { randomID, colorScheme, css } from '../../styles';
import { CloseButton } from '../ActionIcon';
import { Box } from '../Box';
import { Overlay } from '../Overlay';
import { Paper } from '../Paper';
import { OptionalPortal } from '../Portal';
import { Text } from '../Text';
import useStyles from './Modal.styles';
export let use = [], element = undefined, className = '', override = {}, opened = false, title = '', zIndex = 200, overflow = 'outside', withCloseButton = true, overlayOpacity = null, overlayColor = null, overlayBlur = 0, radius = 'sm', size = 'md', transition = 'scale', transitionOptions = { duration: 100, easing: sineInOut }, overlayTransition = 'fade', overlayTransitionOptions = { duration: 200, easing: sineInOut }, closeButtonLabel = 'svelteui-close-button', id = 'svelteui', shadow = 'lg', padding = 'lg', closeOnClickOutside = true, closeOnEscape = true, trapFocus = true, centered = null, target = '#SVELTEUI_PROVIDER', withinPortal = true;
export { className as class };
const dispatch = createEventDispatcher();
const castAny = (self) => self;
const baseId = randomID(id);
const titleId = `${baseId}-title`;
const bodyId = `${baseId}-body`;
const _overlayOpacity = typeof overlayOpacity === 'number' ? overlayOpacity : $colorScheme === 'dark' ? 0.85 : 0.75;
const { handleFocusReturn } = useFocusReturn();
const closeOnEscapePress = (event) => {
    if (!trapFocus && event.code === 'Escape' && closeOnEscape) {
        onClose();
    }
};
function onClose() {
    dispatch('close');
}
// Temporary, just add zIndex to Portal component
const zIndexStyles = css({ zIndex });
$: handleFocusReturn(opened);
$: {
    onMount(() => {
        if (!trapFocus) {
            typeof window !== 'undefined'
                ? window.addEventListener('keydown', closeOnEscapePress)
                : null;
        }
    });
}
$: if (opened && ((typeof target === 'string' && !document.querySelector(target)) || !target)) {
    throw new Error('Wrap your app in the SvelteUIProvider, or provide a sufficent target throught the "target={\'\'}" prop ');
}
$: lockScroll = opened;
$: _transition = getTransition(transition);
$: _overlayTransition = getTransition(overlayTransition);
$: ({ cx, classes, getStyles } = useStyles({ centered, overflow, size, zIndex }, { name: 'Modal' }));
</script>

{#if opened}
	<OptionalPortal class={zIndexStyles()} {withinPortal} {target}>
		<Box
			id={baseId}
			bind:element
			{use}
			{...$$restProps}
			class={cx(className, classes.root, getStyles({ css: override }))}
		>
			<div
				role="presentation"
				class={classes.inner}
				use:lockscroll={lockScroll}
				on:keydown|capture={(event) => {
					const shouldTrigger =
						castAny(event.target)?.getAttribute('data-svelteui-stop-propagation') !== 'true';
					shouldTrigger && event.code === 'Escape' && closeOnEscape && onClose();
				}}
			>
				<div class={classes.transition} transition:_transition={transitionOptions}>
					<Paper
						class={classes.modal}
						{shadow}
						{padding}
						{radius}
						role="dialog"
						aria-labelledby={titleId}
						aria-describedby={bodyId}
						aria-modal
						tabindex={-1}
						use={[[focustrap, trapFocus]]}
					>
						{#if title || withCloseButton}
							<div class={classes.header}>
								<slot name="title">
									<Text id={titleId} class={classes.title}>
										{title}
									</Text>
								</slot>

								{#if withCloseButton}
									<CloseButton
										iconSize={16}
										on:click={onClose}
										aria-label={closeButtonLabel}
										class={classes.close}
									/>
								{/if}
							</div>
						{/if}
						<div id={bodyId} class={classes.body}>
							<slot>Place some content</slot>
						</div>
					</Paper>
				</div>
			</div>
			<div transition:_overlayTransition={overlayTransitionOptions}>
				<Overlay
					class={classes.overlay}
					override={{ position: 'fixed' }}
					zIndex={0}
					on:mousedown={() => closeOnClickOutside && onClose()}
					blur={overlayBlur}
					color={overlayColor || 'black'}
					opacity={_overlayOpacity}
				/>
			</div>
		</Box>
	</OptionalPortal>
{/if}
